@@ -9,6 +9,7 @@ use strict;
 use warnings;
 use Carp 'croak';
 use JSON;
+use Fcntl;
 use IO::Select;
 use IO::Socket::SSL;
 use Time::HiRes qw(time);
@@ -17,15 +18,13 @@ use RPC::Switch::Client::Tiny::Netstring;
 use RPC::Switch::Client::Tiny::Async;
 use RPC::Switch::Client::Tiny::SessionCache;
 
-our $VERSION = '1.66_01';
+our $VERSION = '1.53';
 
 sub new {
 	my ($class, %args) = @_;
 	my $s = $args{sock} or croak __PACKAGE__ . " expects sock";
-	unless ($^O eq 'MSWin32') { # cpantester: strawberry perl does not support blocking() call
-		defined(my $b = $s->blocking()) or croak __PACKAGE__ . " bad socket: $!";
-		unless ($b) { croak __PACKAGE__ . " nonblocking socket not supported"; }
-	}
+	my $f = fcntl($s, F_GETFL, 0) or croak __PACKAGE__ . " bad socket: $!";
+	if ($f & O_NONBLOCK) { croak __PACKAGE__ . " nonblocking socket not supported"; }
 	unless (exists $args{who}) { croak __PACKAGE__ . " expects who"; }
 	my $self = bless {
 		%args,
@@ -35,7 +34,6 @@ sub new {
 		channels  => {}, # open rpcswitch channels
 		methods   => {}, # defined worker methods
 		announced => {}, # announced worker methods
-		msglimit  => 999999, # max netstring size
 	}, $class;
 	if (ref($self->{sock}) eq 'IO::Socket::SSL') {
 		$self->{auth_method} = 'clientcert' unless exists $self->{auth_method};
@@ -43,7 +41,6 @@ sub new {
 	} else {
 		$self->{auth_method} = 'password' unless exists $self->{auth_method};
 	}
-	$self->{json_utf8} = $self->{client_encoding_utf8} ? {} : {utf8 => 1};
 	return $self;
 }
 
@@ -54,13 +51,8 @@ sub rpc_error {
 sub rpc_send {
 	my ($self, $msg) = @_;
 	my $s = $self->{sock};
-	my $len = length($s);
-	if ($self->{msglimit} && ($len > $self->{msglimit})) {
-		warn "rpc_send msglimit exceeded: $len > $self->{msglimit}";
-		return;
-	}
 	$msg->{jsonrpc} = '2.0';
-	my $str = to_json($msg, {canonical => 1, %{$self->{json_utf8}}});
+	my $str = to_json($msg, {canonical => 1});
 	$self->{trace_cb}->('SND', $msg) if $self->{trace_cb};
 	return netstring_write($s, $str);
 }
@@ -352,8 +344,6 @@ sub child_handler {
 	}
 	$self->{sock} = $wr;
 	delete $self->{trace_cb};
-	local $SIG{INT} = 'DEFAULT';
-	local $SIG{PIPE} = 'IGNORE'; # handle sigpipe via print/write result
 
 	# When session handling is enabled a child might process
 	# more than one request with the same session_id.
@@ -365,7 +355,7 @@ sub child_handler {
 			die "worker child: $@" if $@;
 			last; # EOF
 		}
-		my $msg = eval { from_json($b, {%{$self->{json_utf8}}}) };
+		my $msg = eval { from_json($b) };
 		die "worker child: $@" if $@;
 
 		# The client catches all possible die() calls, so that it is
@@ -373,17 +363,13 @@ sub child_handler {
 		#
 		my $params;
 		my $callback = $self->{methods}{$msg->{method}}{cb};
-		my @resp;
-		eval {
-			local $SIG{PIPE} = 'DEFAULT'; # reenable sigpipe for worker code
-			@resp = $callback->($msg->{params}, $msg->{rpcswitch});
-		};
+		my @resp = eval { $callback->($msg->{params}, $msg->{rpcswitch}) };
 		if (my $err = $@) {
 			$params = ['RES_ERROR', $msg->{id}, $err];
 		} else {
 			$params = ['RES_OK', $msg->{id}, @resp];
 		}
-		$b = eval { to_json($params, {%{$self->{json_utf8}}}) };
+		$b = eval { to_json($params) };
 		return 1 if $@; # signal die from json encode
 		my $res = netstring_write($self->{sock}, $b);
 		return 2 unless $res; # signal socket error
@@ -397,7 +383,7 @@ sub child_handler {
 sub _worker_child_write {
 	my ($self, $child, $msg) = @_;
 
-	my $b = to_json($msg, {canonical => 1, %{$self->{json_utf8}}});
+	my $b = to_json($msg, {canonical => 1});
 	my $res = netstring_write($child->{reader}, $b); # forward request to worker child
 	die rpc_error('io', 'netstring_write') unless $res;
 	return;
@@ -408,22 +394,14 @@ sub _worker_child_get {
 
 	# First try to reuse child for existing session
 	#
-	if (my $sessioncache = $self->{sessioncache}) {
+	if ($self->{sessioncache}) {
 		if (my $session_req = $self->is_session_req($msg->{params})) {
-			if (my $child = $sessioncache->session_get($session_req->{id}, $msg->{id}, $msg->{rpcswitch}{vci})) {
+			if (my $child = $self->{sessioncache}->session_get($session_req->{id}, $msg->{id})) {
 				return $child;
 			}
-		} elsif ($sessioncache->{session_persist_user}) {
-			if (exists $msg->{params}{$sessioncache->{session_persist_user}}) {
-				my $user = $msg->{params}{$sessioncache->{session_persist_user}};
-				if (my $child = $sessioncache->session_get_per_user($user, $msg->{id}, $msg->{rpcswitch}{vci})) {
-					delete $child->{session}; # reused session will be added after session_resp
-					return $child;
-				}
-			}
 		}
 	}
-	my $child = $self->{async}->child_start($self, $msg->{id}, $msg->{rpcswitch}{vci});
+	my $child = $self->{async}->child_start($self, $msg->{id});
 	return $child;
 }
 
@@ -435,17 +413,14 @@ sub _worker_childs_dequeue_and_run {
 		my $rpcswitch_resp = rpcswitch_resp($msg->{rpcswitch});
 
 		my $child = eval { $self->_worker_child_get($msg) };
+		unless ($@) {
+			$self->{async}->job_add($child, $msg->{id}, {rpcswitch => $rpcswitch_resp});
+			eval { $self->_worker_child_write($child, $msg) };
+		}
 		if ($@) {
 			$self->rpc_send({id => $id, result => ['RES_ERROR', $@], rpcswitch => $rpcswitch_resp});
 		} else {
-			eval { $self->_worker_child_write($child, $msg) };
-			if ($@) {
-				$self->rpc_send({id => $id, result => ['RES_ERROR', $@], rpcswitch => $rpcswitch_resp});
-				$self->{async}->child_finish($child, 'error');
-			} else {
-				$self->rpc_send({id => $id, result => ['RES_WAIT', $id], rpcswitch => $rpcswitch_resp});
-				$self->{async}->job_add($child, $msg->{id}, {rpcswitch => $rpcswitch_resp});
-			}
+			$self->rpc_send({id => $id, result => ['RES_WAIT', $id], rpcswitch => $rpcswitch_resp});
 		}
 	}
 	return;
@@ -461,25 +436,17 @@ sub _worker_child_read_and_finish {
 		$res = $self->rpc_send({method => 'rpcswitch.result', params => ['RES_ERROR', $child->{id}, $err], rpcswitch => $child->{rpcswitch}});
 		$self->{async}->child_finish($child, 'error');
 	} else {
-		my $params = eval { from_json($b, {%{$self->{json_utf8}}}) };
+		my $params = eval { from_json($b) };
 		if ($@) {
 			$res = $self->rpc_send({method => 'rpcswitch.result', params => ['RES_ERROR', $child->{id}, $@], rpcswitch => $child->{rpcswitch}});
 			$self->{async}->child_finish($child, 'error');
 		} else {
 			$res = $self->rpc_send({method => 'rpcswitch.result', params => $params, rpcswitch => $child->{rpcswitch}});
-			unless ($res) {
-				my $err = "result msg limit exceeded: " . length($b);
-				$res = $self->rpc_send({method => 'rpcswitch.result', params => ['RES_ERROR', $child->{id}, $err], rpcswitch => $child->{rpcswitch}});
-				$self->{async}->child_finish($child, 'error');
-				return $res;
-			}
 
 			if (my $sessioncache = $self->{sessioncache}) {
 				if (my $set_session = $self->is_session_resp($params)) {
-					if (!exists $child->{session} || ($child->{session}{id} ne $set_session->{id})) {
-						$child->{session} = $sessioncache->session_new($set_session);
-						$sessioncache->expire_insert($child->{session});
-					}
+					$child->{session} = $sessioncache->session_new($set_session);
+					$sessioncache->expire_insert($child->{session});
 				}
 
 				if ($sessioncache->session_put($child)) {
@@ -490,14 +457,6 @@ sub _worker_child_read_and_finish {
 						}
 					}
 					$child = undef;
-				} elsif (my $idle_child = $sessioncache->session_get_per_user_idle($child)) {
-					# update idle user session with older session_id
-					#
-					$self->{async}->child_finish($idle_child, 'update');
-
-					if ($sessioncache->session_put($child)) {
-						$child = undef;
-					}
 				}
 			}
 			if ($child) {
@@ -513,7 +472,7 @@ sub _worker_sessions_expire {
 	return unless $self->{sessioncache};
 
 	# If a job for the expired session is active, the session
-	# will be dropped when session_put() is called after the
+	# will be dropped when sesseion_put() is called after the
 	# job completed.
 	#
 	while (my $child = $self->{sessioncache}->expired_dequeue()) {
@@ -586,7 +545,7 @@ sub rpc_handler {
 			die rpc_error('io', $@) if $@;
 			return; # EOF
 		}
-		my $msg = eval { from_json($b, {%{$self->{json_utf8}}}) };
+		my $msg = eval { from_json($b) };
 		die rpc_error('jsonrpc', $@) if $@;
 		$self->{trace_cb}->('RCV', $msg) if $self->{trace_cb};
 		my $res = eval { $handler->($self, $msg, @handler_params) };
@@ -646,9 +605,7 @@ sub work {
 		if (keys %{$async->{finished}}) {
 			local $SIG{ALRM} = sub { warn "worker child wait timeout\n" };
 			alarm(1); # wait at most for one second
-			unless ($async->childs_reap()) { # blocking
-				$async->childs_reap(nonblock => 1); # continue nonblocking after timeout
-			}
+			$async->childs_reap(); # blocking
 			alarm(0);
 		}
 
@@ -785,11 +742,11 @@ of the rpc-switch server.
 
 The implementation is based on the following protocols:
 
-- json-rpc 2.0: L<https://www.jsonrpc.org/specification>
+- json-rpc 2.0: https://www.jsonrpc.org/specification
 
-- netstring proto: L<http://cr.yp.to/proto/netstrings.txt>
+- netstring proto: http://cr.yp.to/proto/netstrings.txt
 
-- rpc-switch: L<https://github.com/a6502/rpc-switch>
+- rpc-switch: https://github.com/a6502/rpc-switch
 
 =head2 Error Handling
 
@@ -813,18 +770,13 @@ on the error type:
 
 =head2 Encoding
 
-All methods expect and return strings in utf8-format, when the option
-client_encoding_utf8 is enabled.
+All methods expect and return strings in utf8-format.
 
 To pass latin1-strings as parameter, a caller would have to convert
 the input first. (see: $utf8 = Encode::encode('utf8', $latin1)).
 
 The json-rpc messages are also utf8-encoded when they are transmitted.
-(see: L<https://metacpan.org/pod/JSON#utf8>)
-
-Without the client_encoding_utf8 option, all passed strings are utf8-encoded
-for the json-rpc transmission, because the client character-encoding is
-assumed to be unknown. This might transmit doubly utf8-encoded strings.
+(see: https://metacpan.org/pod/JSON#utf8)
 
 =head2 Async handling
 
@@ -848,7 +800,7 @@ one request with the same session_id.
 
 The session handling is loosely based on the HTTP Set-Cookie/Cookie
 Mechanism described in RFC 6265, and uses json paramters instead of
-http-headers. (see: L<https://datatracker.ietf.org/doc/html/rfc6265>)
+http-headers. (see: https://datatracker.ietf.org/doc/html/rfc6265)
 
 - the worker signals a set_session request via:
 
@@ -938,8 +890,6 @@ The accepted arguments are:
 
 =item token: token for auth_method 'password' (optional for 'clientcert')
 
-=item client_encoding_utf8: all client strings are in utf8-format
-
 =item timeout: optional rpc-request timeout (defaults to 0 (unlimited))
 
 =item trace_cb: optional handler to log SND/RCV output
@@ -1006,12 +956,6 @@ connection, and there are no outstanding requests ($@ == '' for eval).
 
 =item options->{session_expire}: default session expire time in seconds
 
-=item options->{session_idle}: session idle time in seconds for user session updates
-
-=item options->{session_persist_user}: reuse active user sessions for given user param
-
-=item options->{max_user_session}: limit session cache per optional user field
-
 =back
 
 The options are described in their respective sections unter DESCRIPTION.
@@ -1024,11 +968,11 @@ The valid fields of the $method_definition are:
 
 =over 4
 
-=item cb: method handler (required)
+=item - cb: method handler (required)
 
-=item doc: optional documentation for method
+=item - doc: optional documentation for method
 
-=item filter: optional filter to restrict method subset of params
+=item - filter: optional filter to restrict method subset of params
 
 =back
 
