@@ -7,7 +7,7 @@ use warnings;
 use Time::HiRes qw(time);
 use Time::Local;
 
-our $VERSION = 1.20;
+our $VERSION = 1.15;
 
 sub new {
 	my ($class, %args) = @_;
@@ -16,13 +16,9 @@ sub new {
 		active    => {}, # active async sessions
 		lru       => {}, # lru list for sessions
 		expiring  => [], # sorted session expire list
-		per_user  => {}, # active sessions per user (optional)
 	}, $class;
 	$self->{lru}{prev} = $self->{lru}{next} = $self->{lru};
 	$self->{session_expire} = 60 unless $self->{session_expire};
-	$self->{session_idle} = 1 unless $self->{session_idle};
-	$self->{session_persist_user} = '' unless $self->{session_persist_user};
-	$self->{max_user_session} = 0 unless $self->{max_user_session};
 	return $self;
 }
 
@@ -112,7 +108,7 @@ sub list_del {
 
 sub session_put {
 	my ($self, $child) = @_;
-	my %runtime = (exists $child->{runtime}) ? (runtime => $child->{runtime}) : ();
+	my $runtime = sprintf "%.02f", time() - $child->{start};
 
 	return unless exists $child->{session};
 
@@ -123,77 +119,27 @@ sub session_put {
 	if ($diff < 0) {
 		return; # session expired
 	}
-	if ($self->{max_user_session} && exists $child->{session}{user}) {
-		my $user = $child->{session}{user};
-		if (exists $self->{per_user}{$user}) {
-			my $cnt = scalar keys %{$self->{per_user}{$user}};
-			if ($cnt >= $self->{max_user_session}) {
-				return; # too many user sessions
-			}
-		}
-		$self->{per_user}{$user}{$child->{session}{id}} = 1;
-	}
-	$self->{trace_cb}->('PUT', {pid => $child->{pid}, id => $child->{id}, session => $child->{session}{id}, %runtime}) if $self->{trace_cb};
+	$self->{trace_cb}->('PUT', {pid => $child->{pid}, id => $child->{id}, session => $child->{session}{id}, runtime => $runtime}) if $self->{trace_cb};
 	$self->{active}{$child->{session}{id}} = $child;
 	list_add($self->{lru}{prev}, $child);
-	delete $child->{runtime};
 	return 1;
 }
 
 sub session_get {
-	my ($self, $session_id, $msg_id, $msg_vci) = @_;
+	my ($self, $session_id, $msg_id) = @_;
 	my %id = (defined $msg_id) ? (id => $msg_id) : ();
-	my %vci = (defined $msg_vci) ? (vci => $msg_vci) : ();
 
 	if (exists $self->{active}{$session_id}) {
 		my $child = delete $self->{active}{$session_id};
 		list_del($child);
 
-		if ($self->{max_user_session} && exists $child->{session}{user}) {
-			my $user = $child->{session}{user};
-			if (exists $self->{per_user}{$user}) {
-				delete $self->{per_user}{$user}{$child->{session}{id}};
-			}
-		}
-
 		my $stoptime = sprintf "%.02f", time() - $child->{start};
-		$self->{trace_cb}->('GET', {pid => $child->{pid}, %id, %vci, session => $session_id, stoptime => $stoptime}) if $self->{trace_cb};
+		$self->{trace_cb}->('GET', {pid => $child->{pid}, %id, session => $session_id, stoptime => $stoptime}) if $self->{trace_cb};
 		return $child;
 	}
 	return;
 }
 
-sub session_get_per_user {
-	my ($self, $user, $msg_id, $msg_vci) = @_;
-
-	if ($self->{max_user_session}) {
-		foreach my $session_id (keys %{$self->{per_user}{$user}}) {
-			return $self->session_get($session_id, $msg_id, $msg_vci);
-		}
-	}
-	return;
-}
-
-sub session_get_per_user_idle {
-	my ($self, $child) = @_;
-
-	return unless exists $child->{session};
-
-	if ($self->{max_user_session} && exists $child->{session}{user}) {
-		my $user = $child->{session}{user};
-		foreach my $session_id (keys %{$self->{per_user}{$user}}) {
-			if (exists $self->{active}{$session_id}) {
-				my $other_child = $self->{active}{$session_id};
-				my $idle = time() - $other_child->{start};
-				if ($idle >= $self->{session_idle}) {
-					return $self->session_get($session_id);
-				}
-			}
-		}
-	}
-	return;
-}
-
 sub parse_isotime {
 	my ($isotime) = @_;
 	my ($yy,$mm,$dd,$h,$m,$s,$msec) = $isotime =~
@@ -213,9 +159,7 @@ sub session_new {
 	}
 	# set default expire in seconds
 	$expiretime = time() + $self->{session_expire} unless $expiretime;
-	my $session = {id => $set_session->{id}, expiretime => $expiretime};
-	$session->{user} = $set_session->{user} if exists $set_session->{user};
-	return $session;
+	return {id => $set_session->{id}, expiretime => $expiretime};
 }
 
 sub lru_list {
